@@ -1,9 +1,4 @@
-//use std::cmp::max;
 
-use crate::discovery::data_types::topic_data::DiscoveredWriterData;
-use crate::discovery::data_types::topic_data::DiscoveredReaderData;
-use crate::discovery::data_types::spdp_participant_data::SPDPDiscoveredParticipantData;
-use crate::discovery::discovery::Discovery;
 
 use log::{debug, error, info, warn, trace};
 use mio::{Poll, Event, Events, Token, Ready, PollOpt};
@@ -23,6 +18,10 @@ use crate::network::udp_listener::UDPListener;
 use crate::network::constant::*;
 use crate::structure::guid::{GuidPrefix, GUID, EntityId, TokenDecode};
 use crate::structure::entity::RTPSEntity;
+use crate::discovery::data_types::topic_data::DiscoveredWriterData;
+use crate::discovery::data_types::topic_data::DiscoveredReaderData;
+//use crate::discovery::data_types::spdp_participant_data::SPDPDiscoveredParticipantData;
+use crate::discovery::discovery::Discovery;
 
 use crate::{
   common::timed_event_handler::{TimedEventHandler},
@@ -73,6 +72,7 @@ pub struct DPEventLoop {
   writers: HashMap<EntityId, Writer>,
 
   discovery_update_notification_receiver: mio_channel::Receiver<DiscoveryNotificationType>,
+
 }
 
 impl DPEventLoop {
@@ -167,6 +167,7 @@ impl DPEventLoop {
       )
       .expect("Failed to register reader update notification.");
 
+
     DPEventLoop {
       domain_info,
       poll,
@@ -275,9 +276,17 @@ impl DPEventLoop {
                 .map( |reader| reader.process_command() )
                 .unwrap_or_else(|| error!("Event for unknown reader {:?}",eid));
             } else if eid.kind().is_writer() {
-              ev_wrapper.writers.get_mut( &eid )
-                .map( |writer| writer.process_writer_command() )
-                .unwrap_or_else(|| error!("Event for unknown writer {:?}",eid));
+              let local_readers =
+                match ev_wrapper.writers.get_mut( &eid ) {
+                  None => { error!("Event for unknown writer {:?}",eid); vec![] },
+                  Some(writer) => {
+                    // Writer will record data to DDSCache and send it out.
+                    writer.process_writer_command();
+                    writer.get_local_readers()
+                  }
+                };
+              // Notify local (same participant) readers that new data is available in the cache.
+              ev_wrapper.message_receiver.notify_data_to_readers(local_readers);
             } else { 
               error!("Entity Event for unknown EntityKind {:?}",eid); 
             },
@@ -453,12 +462,14 @@ impl DPEventLoop {
   }
 
   fn update_participant(&mut self, participant_guid_prefix: GuidPrefix ) {
-    info!("update_participant - begin for {:?}", participant_guid_prefix);
+    debug!("update_participant - begin for {:?}", participant_guid_prefix);
+
     if participant_guid_prefix == self.domain_info.domain_participant_guid.guidPrefix {
-      // Our own participant was updated (initialized.)
-      // What should we do now?
-      debug!("Own participant initialized");
-    } else {
+      info!("Own participant update");
+    } 
+
+
+    {
       let db = self.discovery_db.read().unwrap();
       // new Remote Participant discovered
       let discovered_participant = 
@@ -570,7 +581,7 @@ impl DPEventLoop {
         }
       } // for
     } // if
-    info!("update_participant - finished for {:?}", participant_guid_prefix);
+    debug!("update_participant - finished for {:?}", participant_guid_prefix);
   } // fn 
 
   fn remote_participant_lost(&mut self, participant_guid_prefix: GuidPrefix ) {
@@ -604,19 +615,6 @@ impl DPEventLoop {
     }
   }
 
-  fn update_discovery_reader(reader: &mut Reader, dpd: &SPDPDiscoveredParticipantData,
-    entity_id: EntityId, expected_endpoint: u32 ) 
-  {
-    debug!("update_discovery_reader - {:?}", reader.topic_name() );
-    if dpd.available_builtin_endpoints.contains(expected_endpoint) {
-      reader.update_writer_proxy( 
-        dpd.as_writer_proxy(true, Some(entity_id)),
-        Discovery::publisher_qos()
-       );
-      debug!("update_discovery reader - endpoint {:?} - {:?}", expected_endpoint, dpd.participant_guid);
-    }
-  }
-
   fn remote_writer_discovered(&mut self, dwd: DiscoveredWriterData) {
     for reader in self.message_receiver.available_readers.values_mut() {
       if &dwd.publication_topic_data.topic_name == reader.topic_name() {
