@@ -160,9 +160,6 @@ impl MessageReceiver {
   // }
 
   pub fn handle_received_packet(&mut self, msg_bytes: Bytes) {
-    self.reset();
-    self.dest_guid_prefix = self.own_guid_prefix;
-
     // Check for RTPS ping message. At least RTI implementation sends these.
     // What should we do with them? The spec does not say.
     if msg_bytes.len() < RTPS_MESSAGE_HEADER_SIZE {
@@ -189,6 +186,15 @@ impl MessageReceiver {
       }
     };
 
+    // And process message
+    self.handle_parsed_message(rtps_message)
+  }
+
+  // This is also called directly from dp_event_loop in case of loopback messages.
+  pub fn handle_parsed_message(&mut self, rtps_message: Message)
+  {
+    self.reset();
+    self.dest_guid_prefix = self.own_guid_prefix;
     self.source_guid_prefix = rtps_message.header.guid_prefix;
 
     for submessage in rtps_message.submessages {
@@ -201,11 +207,11 @@ impl MessageReceiver {
   }
 
   fn handle_entity_submessage(&mut self, submessage: EntitySubmessage) {
-    if self.dest_guid_prefix != self.own_guid_prefix {
-      debug!("Messages are not for this participant?");
-      debug!("dest_guid_prefix: {:?}", self.dest_guid_prefix);
-      debug!("participant guid: {:?}", self.own_guid_prefix);
-      return; // Wrong target received
+    if self.dest_guid_prefix != self.own_guid_prefix 
+        && self.dest_guid_prefix != GuidPrefix::GUIDPREFIX_UNKNOWN {
+      debug!("Message is not for this participant. Dropping. dest_guid_prefix={:?} participant guid={:?}", 
+        self.dest_guid_prefix, self.own_guid_prefix);
+      return 
     }
 
     let mr_state = self.give_message_receiver_info();
@@ -227,7 +233,7 @@ impl MessageReceiver {
                             )
                       )
           {
-            trace!("handle_entity_submessage DATA from unknown handling in {:?}",&reader);
+            debug!("handle_entity_submessage DATA from unknown handling in {:?}",&reader);
             reader.handle_data_msg(data.clone(), data_flags, mr_state.clone());
           }
         } else {
@@ -333,6 +339,13 @@ impl MessageReceiver {
     }
   }
 
+  pub fn notify_data_to_readers(&self, readers: Vec<EntityId>) {
+    for eid in readers {
+      self.available_readers.get(&eid)
+        .map( |r| r.notify_cache_change() );
+    }
+  }
+
   // sends 0 seqnum acknacks for those writer that haven't had any action
   pub fn send_preemptive_acknacks(&mut self) {
     for reader in self.available_readers.values_mut() {
@@ -477,6 +490,7 @@ use super::*;
     let _serializedPayload = to_bytes::<ShapeType, LittleEndian>(&deserializedShapeType);
     let (_dwcc_upload, hccc_download) = mio_channel::channel::<WriterCommand>();
     let (status_sender, _status_receiver) = mio_channel::sync_channel(10);
+
     let mut _writerObject = Writer::new(
       GUID::new_with_prefix_and_id(guiPrefix, EntityId::createCustomEntityID([0, 0, 2], EntityKind::WRITER_WITH_KEY_USER_DEFINED)),
       hccc_download,
