@@ -4,7 +4,7 @@ use crate::messages::submessages::submessages::*;
 
 use speedy::{Context, Writer, Writable};
 
-#[derive(Debug, PartialEq)]
+#[derive(Debug, PartialEq, Clone)]
 pub struct SubMessage {
   pub header: SubmessageHeader,
   pub body: SubmessageBody,
@@ -12,7 +12,7 @@ pub struct SubMessage {
 
 // TODO: Submessages should implement some Length trait that returns the length of
 // Submessage in bytes. This is needed for Submessage construction.
-#[derive(Debug, PartialEq)]
+#[derive(Debug, PartialEq, Clone)]
 pub enum SubmessageBody {
   Entity(EntitySubmessage),
   Interpreter(InterpreterSubmessage),
