@@ -38,6 +38,11 @@ impl SequenceNumber {
       SequenceNumber::from(0)
     }
   }
+
+  pub fn range_inclusive(begin:Self, end:Self) -> SequenceNumberRange
+  {
+    SequenceNumberRange::new(begin,end)
+  }
 }
 
 impl From<i64> for SequenceNumber {
@@ -64,6 +69,30 @@ impl From<SequenceNumber> for i64 {
   }
 }
 
+pub struct SequenceNumberRange {
+  begin: SequenceNumber,
+  end: SequenceNumber,
+}
+
+impl SequenceNumberRange {
+  pub fn new(begin:SequenceNumber, end:SequenceNumber) -> SequenceNumberRange {
+    SequenceNumberRange { begin, end }
+  }
+}
+
+impl Iterator for SequenceNumberRange {
+  type Item = SequenceNumber;
+  fn next(&mut self) -> Option<Self::Item> {
+    if self.begin > self.end {
+      None
+    } else {
+      let b = self.begin;
+      self.begin = b + SequenceNumber::new(1);
+      Some(b)
+    }
+  }
+}
+
 mod sequence_number_checked {
   use super::*;
   checked_impl!(CheckedAdd, checked_add, SequenceNumber);
@@ -233,10 +262,14 @@ where N: Clone + Copy + Debug + Hash + PartialEq + Eq + NumOps +  From<i64>  + O
           error!("from_base_and_set : need base <= set start: base={:?} and set {:?}",
             base, set); 
         }
-        if i64::from(end - start) > 256 {
-          error!("from_base_and_set : max size (256) exceeded, start = {:?} end = {:?}",
-            start,end);  
-        }
+        let end = 
+          if i64::from(end - start) > 256 {
+            let truncated_end = start + N::from(256);
+            error!("from_base_and_set : max size (256) exceeded, start = {:?} end = {:?}. Truncating end to {:?}",
+              start, end, truncated_end );
+            truncated_end
+          } else { end }; // sanity ok.
+        // TODO: Sanity check that (end - base) <= 256
         // work:
         let num_bits =  i64::from(end - start + N::from(1));
         let mut sns = NumberSet::<N>::new(
@@ -255,6 +288,7 @@ where N: Clone + Copy + Debug + Hash + PartialEq + Eq + NumOps +  From<i64>  + O
     NumberSetIter::<N> {
       seq: self,
       at_bit: 0,
+      rev_at_bit: self.num_bits,
     }
   }
 }
@@ -302,11 +336,13 @@ impl<C: Context, N > Writable<C> for NumberSet<N>
   }
 }
 
+#[derive(Clone, Debug, Hash, PartialEq, Eq)]
 pub struct NumberSetIter<'a,N>
   where N: Clone + Debug + Hash + PartialEq + Eq + NumOps +  From<i64> + Ord + PartialOrd
 {
   seq: &'a NumberSet<N>,
   at_bit: u32,
+  rev_at_bit: u32,
 }
 
 impl<'a,N> Iterator for NumberSetIter<'_,N>
@@ -318,7 +354,7 @@ impl<'a,N> Iterator for NumberSetIter<'_,N>
     //TODO: This probably could made faster with the std function
     // .leading_zeroes() in type u32 to do several iterations of the loop in
     // one step, given that we have clz as a machine instruction or short sequence.
-    while self.at_bit < self.seq.num_bits {
+    while self.at_bit < self.rev_at_bit {
       // bit indexing formula from RTPS spec v2.3 Section 9.4.2.6
       let have_one = 
         self.seq.bitmap[(self.at_bit / 32) as usize] 
@@ -333,6 +369,25 @@ impl<'a,N> Iterator for NumberSetIter<'_,N>
     None
   }
 }
+impl<'a,N> DoubleEndedIterator for NumberSetIter<'_,N>
+  where N: Clone + Copy + Debug + Hash + PartialEq + Eq + NumOps +  From<i64> + Ord + PartialOrd
+{
+  fn next_back(&mut self) -> Option<Self::Item> {
+    while self.at_bit < self.rev_at_bit {
+      // bit indexing formula from RTPS spec v2.3 Section 9.4.2.6
+      self.rev_at_bit -= 1;
+      let have_one = 
+        self.seq.bitmap[(self.rev_at_bit / 32) as usize] 
+          & (1 << (31 - self.rev_at_bit % 32))
+        != 0;
+      if have_one { 
+        return Some( N::from(self.rev_at_bit as i64) 
+                      + self.seq.bitmap_base )  
+      } 
+    }
+    None
+  }
+}
 
 #[cfg(test)]
 mod tests {
@@ -398,6 +453,28 @@ mod tests {
             0x00, 0x00, 0x00, 0x2A,
             0x00, 0x00, 0x00, 0x00]
   },
+  {
+    sequence_number_set_one,
+    (|| {
+          let mut set = SequenceNumberSet::new(SequenceNumber::from(1),1);
+          set.insert(SequenceNumber::from(1));
+          set
+      })(),
+      le = [0x00, 0x00, 0x00, 0x00, // bitmapBase high
+            0x01, 0x00, 0x00, 0x00, // bitmapBase low
+            0x01, 0x00, 0x00, 0x00, // bitmap bit count
+            0x00, 0x00, 0x00, 0x80], // bitmap word 0
+      // The last word here has 11 bits set from MSB end, and then 25-11 = 14 zeroes.
+      // The last 32-25 = 7 bits are undefined. Our implementation sets them to zero,
+      // but others may set to ones. 
+      // 0xffc0_00YZ, where  YZ & 0x80 = 0x00, but otherwise undefined
+      // So e.g. 0x7f is valid least siginifanct byte, as is 0x00, or 0x0f.
+      be = [0x00, 0x00, 0x00, 0x00,
+            0x00, 0x00, 0x00, 0x01,
+            0x00, 0x00, 0x00, 0x01,
+            0x80, 0x00, 0x00, 0x00]
+
+  },
   {
       sequence_number_set_manual,
       (|| {
@@ -420,6 +497,28 @@ mod tests {
             0x00, 0x00, 0x00, 0x01,
             0x00, 0x00, 0x00, 0x19,
             0xff, 0xc0, 0x00, 0x00]
+  },
+  {
+      sequence_number_set_multiword,
+      (|| {
+          // send sequence numbers 10,11,12,..,52. (43 ones) Set goes up to 64.
+          let mut set = SequenceNumberSet::new(SequenceNumber::from(10),64);
+          for sn in 10..=52 {
+            set.insert(SequenceNumber::from(sn));
+          }
+          set
+      })(),
+      le = [0x00, 0x00, 0x00, 0x00,
+            0x0A, 0x00, 0x00, 0x00, // base = 10
+            0x40, 0x00, 0x00, 0x00, // 0x40 = 64 bits in map => 2 words
+            0xff, 0xff, 0xff, 0xff, // bits 10..41 are all ones
+            0x00, 0x00, 0xe0, 0xff, // bits 42..51 (10 pcs) are ones, rest zero
+            ],
+      be = [0x00, 0x00, 0x00, 0x00,
+            0x00, 0x00, 0x00, 0x0A,
+            0x00, 0x00, 0x00, 0x40,
+            0xff, 0xff, 0xff, 0xff,
+            0xff, 0xe0, 0x00, 0x00]
   });
 
 
